﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using ThinkGeo.MapSuite.WpfDesktopEdition;
using ThinkGeo.MapSuite.Core;
using System.Diagnostics;

namespace LearnWPF
{
	/// <summary>
	/// Interaction logic for WpfMAp.xaml
	/// </summary>
	/// 
	public partial class WpfMAp : Window
	{
		LayerOverlay deviceOverlay = new LayerOverlay();
		public WpfMAp()
		{
			InitializeComponent();
			this.Loaded += new RoutedEventHandler(WpfMAp_Loaded);
			wpfMap.MapClick += new EventHandler<MapClickWpfMapEventArgs>(wpfMap_MapClick);

			
		}

		void wpfMap_MapClick(object sender, MapClickWpfMapEventArgs e)
		{
			
			
		}

		void wpfMap_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
		{
			
		}

		void WpfMAp_Loaded(object sender, RoutedEventArgs e)
		{
			LoadMap();
		}

		private void LoadMap()
		{
			GdiPlusRasterLayer rootLayer = new GdiPlusRasterLayer("root.jpg");
			LayerOverlay ovrlay = new LayerOverlay();
			
			ovrlay.Layers.Add(rootLayer);
			rootLayer.Open();			
			//wpfMap.Height = 600;
			//wpfMap.Width = 1000;
			wpfMap.CurrentExtent = rootLayer.GetBoundingBox();

			double maxScale = ExtentHelper.GetScale(wpfMap.CurrentExtent, (float)wpfMap.ActualWidth, wpfMap.MapUnit);

			// Modify 3 to any other value which means how many level lower than the maximum scale
			//double minScale = maxScale / Math.Pow(2, 10);
			double minScale = ExtentHelper.GetScale(wpfMap.CurrentExtent, (float)wpfMap.ActualWidth, wpfMap.MapUnit,10);

			wpfMap.MinimumScale = minScale;
			wpfMap.MaximumScale = maxScale;

			rootLayer.Close();			

			wpfMap.MapTools.Logo.IsEnabled = false;
			ovrlay.TransitionEffect = TransitionEffect.None;
			ovrlay.TileType = TileType.MultipleTile;

			wpfMap.Overlays.Add("root", ovrlay);
			
			
		}			

		private void DrawObjct()
		{
			PopupOverlay popupOverlay = new PopupOverlay();
			
			
			InMemoryFeatureLayer objectFeaturelayer = wpfMap.EditOverlay.EditShapesLayer;
			
			objectFeaturelayer.Open();
			objectFeaturelayer.Columns.Clear();
			objectFeaturelayer.Columns.Add(new FeatureSourceColumn("Type"));
			objectFeaturelayer.Columns.Add(new FeatureSourceColumn("Id"));
			objectFeaturelayer.Columns.Add(new FeatureSourceColumn("Index"));

			objectFeaturelayer.Close();

			ValueStyle valueStyle = new ValueStyle();
			valueStyle.ColumnName = "Id";

			objectFeaturelayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.IsActive = false;
			objectFeaturelayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.IsActive = false;
			objectFeaturelayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.IsActive = false;

			

			for (int i = 0; i < 20; i++)
			{
				Random rand = new Random(i);
				PointShape pointShape = new PointShape(-122.5 + rand.NextDouble(), 36.5 + rand.NextDouble());

				Feature objectFeature = new Feature(pointShape);

				objectFeature.ColumnValues.Add("Type", i.ToString());
				objectFeature.ColumnValues.Add("Id", i.ToString());
				objectFeature.ColumnValues.Add("Index", i.ToString());
				PointStyle objectPointStyle = new PointStyle(new GeoImage(System.IO.Path.GetFullPath("Pin.png")));
				objectPointStyle.PointType = PointType.Bitmap;

				valueStyle.ValueItems.Add(new ValueItem(i.ToString(), objectPointStyle));
				//objectFeaturelayer.InternalFeatures.Add(objectFeature);

				Popup popup = new Popup(pointShape);
				popup.Content = pointShape.X + "," + pointShape.Y;
				popupOverlay.Popups.Add(popup);
			}

			wpfMap.Overlays.Add(popupOverlay);


			objectFeaturelayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(valueStyle);
			objectFeaturelayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

			wpfMap.EditOverlay.CanDrag = true;
			wpfMap.EditOverlay.CalculateAllControlPoints();

			//wpfMap.EditOverlay.EditShapesLayer = objectFeaturelayer;

			//wpfMap.Refresh(wpfMap.EditOverlay);

			//deviceOverlay.TransitionEffect = TransitionEffect.None;

			//deviceOverlay.Layers.Clear();
			//deviceOverlay.Layers.Add("ObjectsLayer", objectFeaturelayer);

			//wpfMap.Overlays.Add("Objects", deviceOverlay);
			wpfMap.EditOverlay.Refresh();
		}	

		private void DrawPolygon()
		{
			wpfMap.EditOverlay = new EditInteractiveOverlay();

			PointShape point = wpfMap.CurrentExtent.GetCenterPoint();

			ScreenPointF screenpoint = ExtentHelper.ToScreenCoordinate(wpfMap.CurrentExtent, point, (float)this.Width, (float)this.Height);

			Feature rectangle = new Feature(GetPolyGonShape(ConvertToWorldPoint(screenpoint.X, screenpoint.Y, Width, Height), ConvertToWorldPoint(screenpoint.X + 60, screenpoint.Y + 60, Width, Height)));
			wpfMap.EditOverlay.EditShapesLayer.InternalFeatures.Add(rectangle);
			wpfMap.EditOverlay.EditShapesLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
			wpfMap.EditOverlay.CanAddVertex = true;
			wpfMap.EditOverlay.CanDrag = true;
			wpfMap.EditOverlay.CanRemoveVertex = true;
			wpfMap.EditOverlay.CanResize = true;
			wpfMap.EditOverlay.CanRotate = true;

			wpfMap.EditOverlay.CalculateAllControlPoints();
			wpfMap.EditOverlay.RotateControlPointsLayer.IsVisible = false;
		}

		private PointShape ConvertToWorldPoint(float x, float y, double Width, double Height)
		{
			return ExtentHelper.ToWorldCoordinate(wpfMap.CurrentExtent, x, y, (float)this.Width, (float)this.Height);

		}

		private PolygonShape GetPolyGonShape(PointShape p1, PointShape p2)
		{

			RingShape ringShape = new RingShape();

			Vertex vertex1 = new Vertex(p1.X, p1.Y);
			Vertex vertex2 = new Vertex(p1.X, p2.Y);
			Vertex vertex4 = new Vertex(p2.X, p2.Y);
			Vertex vertex3 = new Vertex(p2.X, p1.Y);
			Vertex vertex5 = new Vertex(p1.X, p1.Y);

			ringShape.Vertices.Add(vertex1);
			ringShape.Vertices.Add(vertex2);
			ringShape.Vertices.Add(vertex3);
			ringShape.Vertices.Add(vertex4);
			ringShape.Vertices.Add(vertex5);

			return new PolygonShape(ringShape);
		}

		private void btnMapClick_Click(object sender, RoutedEventArgs e)
		{
			//DrawPolygon();
			//wpfMap.Refresh(wpfMap.EditOverlay);
			DrawObjct();
		}
	}
}
